require 'fileutils'
require 'log'

class FileController

	attr_reader :op4_mas_helper_log
	attr_reader :op4_mas_app_log
	attr_reader :op4_mas_preferences_output

	attr_reader :op4_web_helper_log
	attr_reader :op4_web_app_log
	attr_reader :op4_web_preferences_output

	attr_reader :system_log
	attr_reader :running_processes_output

	attr_reader :op39_helper_log
	attr_reader :op39_app_log
	
	attr_reader :op38_agent_log
	attr_reader :op38_app_log
	
	def initialize(reportFolder)
		@homefolder = File.expand_path("~/")
		@reportFolder = reportFolder

		self.process_38
		self.process_39
		self.process_40_web
		self.process_40_mas
		@system_log = process_systemlog
		self.process_crashes
		@op4_web_preferences_output = process_preferences_web
		@op4_mas_preferences_output = process_preferences_mas
		@running_processes_output = generate_running_processes
	end

	def process_log(logPath, destPath="")

		filename = logPath.split('/')[-1]
		filename.sub!(/crash$/, "txt")
		filename.sub!(/log$/, "txt")

		# Check if log exists
		if File.exist?(logPath) then

			fullDestPath = "#{@reportFolder}/#{destPath}"
			
			# Create destination path, unless it already exists
			unless File.exist?(fullDestPath)
				Log.instance.info("[#{destPath}] Creating: #{fullDestPath}")
				Dir.mkdir(fullDestPath)
			end

			# Copy log to destination path
			Log.instance.info("[#{destPath}] Copying 4000 lines of #{logPath}")
			#FileUtils.cp(logPath, "#{fullDestPath}/#{filename}")
			%x{/usr/bin/tail -n 4000 "#{logPath}" > "#{fullDestPath}/#{filename}"}
		end

		if File.exist?("#{fullDestPath}/#{filename}")
			return true
		else
			return false
		end

	end

	def process_38
		#
		# Copy 1Password 3.8 Files
		op38log = "#{@homefolder}/Library/Logs/1Password/1Password.log"
		op38Agentlog = "#{@homefolder}/Library/Logs/1Password/1PasswordAgent.log"


		@op38_app_log = process_log(op38log, "OP38")
		@op38_agent_log = process_log(op38Agentlog, "OP38")
	end

	def process_39
		#
		# Copy 1Password 3.9 Files
		op39log = "#{@homefolder}/Library/Containers/com.agilebits.onepassword-osx/Data/Library/Logs/1Password.log" 
		op39Helperlog = "#{@homefolder}/Library/Containers/com.agilebits.onepassword-osx-helper/Data/Library/Logs/1Password Helper.log" 

		@op39_app_log = process_log(op39log, "OP39")
		@op39_helper_log = process_log(op39Helperlog, "OP39")
	end

	def process_40_web
		op40log = "#{@homefolder}/Library/Logs/1Password/com.agilebits.onepassword4.log"
		op40Helperlog = "#{@homefolder}/Library/Logs/1Password/2BUA8C4S2C.com.agilebits.onepassword4-helper.log" 
		
		fullDestPath = "#{@reportFolder}/OP40_web"
			
		return unless File.exists?(op40log) or File.exists?(op40Helperlog)

		# Create destination path, unless it already exists
		unless File.exist?(fullDestPath)
			Log.instance.info("[OP4 Web] Creating: #{fullDestPath}")
			Dir.mkdir(fullDestPath)
		end

		@op4_web_app_log = process_40_web_app_log(op40log)
		@op4_web_helper_log = process_40_web_helper_log(op40Helperlog)
	end

	def process_40_web_app_log(op40log)
		fullDestPath = "#{@reportFolder}/OP40_web"

		# Check if app log exists
		if File.exist?(op40log)
			# Copy log to destination path
			Log.instance.info("[OP4 Web] Copying 4000 lines of #{op40log}")
			allLogs = Dir["#{HOME}/Library/Logs/1Password/com.agilebits.onepassword4*"]

			if allLogs.count > 1
				%x{/usr/bin/tail -n 4000 "#{allLogs[1]}" | cat - "#{allLogs[0]}" | /usr/bin/tail -n 4000 > "#{fullDestPath}/com.agilebits.onepassword4.txt"}
			else
				%x{/usr/bin/tail -n 4000 "#{op40log}" > "#{fullDestPath}/com.agilebits.onepassword4.txt"}
			end
		end

		if File.exist?("#{fullDestPath}/com.agilebits.onepassword4.txt")
			return true
		else
			return false
		end
	end

	def process_40_web_helper_log(op40Helperlog)
		fullDestPath = "#{@reportFolder}/OP40_web"

		# Check if app log exists
		if File.exist?(op40Helperlog)

			fullDestPath = "#{@reportFolder}/OP40_web"
			
			# Copy log to destination path
			Log.instance.info("[OP4 Web] Copying 4000 lines of #{op40Helperlog}")
			allLogs = Dir["#{HOME}/Library/Logs/1Password/2BUA8C4S2C.com.agilebits.onepassword4-helper.*"]

			if allLogs.count > 1
				%x{/usr/bin/tail -n 4000 "#{allLogs[1]}" | cat - "#{allLogs[0]}" | /usr/bin/tail -n 4000 > "#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword4-helper.txt"}
			else
				%x{/usr/bin/tail -n 4000 "#{op40Helperlog}" > "#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword4-helper.txt"}
			end
		end

		if File.exist?("#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword4-helper.txt")
			return true
		else
			return false
		end
	end

	def process_40_mas
		op40log = "#{@homefolder}/Library/Containers/com.agilebits.onepassword-osx/Data/Library/Logs/1Password/com.agilebits.onepassword-osx.log"  
		op40Helperlog = "#{@homefolder}/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Logs/1Password/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.log" 

		fullDestPath = "#{@reportFolder}/OP40_mas"
			
		return unless File.exists?(op40log) or File.exists?(op40Helperlog)

		# Create destination path, unless it already exists
		unless File.exist?(fullDestPath)
			Log.instance.info("[OP4 MAS] Creating: #{fullDestPath}")
			Dir.mkdir(fullDestPath)
		end

		@op4_mas_app_log = process_40_mas_app_log(op40log)
		@op4_mas_helper_log = process_40_mas_helper_log(op40Helperlog)
	end
	
	def process_40_mas_app_log(op40log)
		fullDestPath = "#{@reportFolder}/OP40_mas"

		# Check if app log exists
		if File.exist?(op40log)
			# Copy log to destination path
			Log.instance.info("[OP4 MAS] Copying 4000 lines of #{op40log}")
			allLogs = Dir["#{HOME}/Library/Containers/com.agilebits.onepassword-osx/Data/Library/Logs/1Password/com.agilebits.onepassword-osx*"]

			if allLogs.count > 1
				%x{/usr/bin/tail -n 4000 "#{allLogs[1]}" | cat - "#{allLogs[0]}" | /usr/bin/tail -n 4000 > "#{fullDestPath}/com.agilebits.onepassword-osx.txt"}
			else
				%x{/usr/bin/tail -n 4000 "#{op40log}" > "#{fullDestPath}/com.agilebits.onepassword-osx.txt"}
			end
		end

		if File.exist?("#{fullDestPath}/com.agilebits.onepassword-osx.txt")
			return true
		else
			false
		end
	end

	def process_40_mas_helper_log(op40Helperlog)
		fullDestPath = "#{@reportFolder}/OP40_mas"

		# Check if app log exists
		if File.exist?(op40Helperlog)

			# Copy log to destination path
			Log.instance.info("[OP4 MAS] Copying 4000 lines of #{op40Helperlog}")
			allLogs = Dir["#{HOME}/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Logs/1Password/2BUA8C4S2C.com.agilebits.onepassword-osx-helper*"]

			if allLogs.count > 1
				%x{/usr/bin/tail -n 4000 "#{allLogs[1]}" | cat - "#{allLogs[0]}" | /usr/bin/tail -n 4000 > "#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.txt"}
			else
				%x{/usr/bin/tail -n 4000 "#{op40Helperlog}" > "#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.txt"}
			end
		end

		if File.exist?("#{fullDestPath}/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.txt")
			return true
		else
			return false
		end
	end

	def process_systemlog
		if File.readable?("/var/log/system.log") then
			Log.instance.info("[System] Copying 1000 lines of /var/log/system.log")
			#FileUtils.cp("/var/log/system.log", "#{@reportFolder}")
			%x{/usr/bin/tail -n 4000 /var/log/system.log > "#{@reportFolder}/system.txt"}
		else
			Log.instance.warn("[System] Error: Unable to copy system.log")
		end

		if File.exist?("#{@reportFolder}/system.txt")
			return true
		else
			return false
		end
	end

	def process_crashes
		Log.instance.info("[System] Processing Crash Reports")
		# Get version of Mac OS X
    /10\.(\d)/.match(`sw_vers`)

		os_version = Integer($1)

		# 8 = Mountain Lion and later
    # else = Lion and earlier
    # 
    if os_version >= 8
        crashFolder = "#{@homefolder}/Library/Logs/DiagnosticReports"
    else
        crashFolder = "#{@homefolder}/Library/Logs/CrashReporter"
    end

    if File.exists?(crashFolder)
      Dir.foreach(crashFolder) do |log|
        if log =~ /securityd/ || log =~ /Safari/ || log =~ /irefox/ || log =~ /1Password/ || log =~ /WebKit/ || log =~ /onepassword/
					process_log("#{crashFolder}/#{log}", "Crashes") if should_copy?("#{crashFolder}/#{log}") && log !~ /^\./
        end
      end
    end

	end

	def should_copy?(filename)
    result = false
    
    f = File.new(filename)
    begin
      t = Time.now-5184000 # 60*24*60*60 Days*Hours*Minutes*Seconds
      n = f.mtime <=> t
      result = (n >= 0)
    rescue
      result = false
    ensure
      return result
    end
  end

	def process_preferences_web
		output = String.new
		output << "Website: Main App Preferences \n\n"

		output << %x{defaults read com.agilebits.onepassword4}

		output << "\n\nWebsite: Helper Preferences \n\n"

		output << %x{defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper}

		file = File.new("#{@reportFolder}/preferences-web.txt", "w+")

		file.write(output)

		file.close

		if File.exist?("#{@reportFolder}/preferences-web.txt")
			return true
		else
			return false
		end
	end

	def process_preferences_mas
		output = String.new
		output << "MAS: Main App Preferences \n\n"

		output << %x{defaults read com.agilebits.onepassword-osx}

		output << "\n\nMAS: Helper Preferences \n\n"

		output << %x{defaults read 2BUA8C4S2C.com.agilebits.onepassword-osx-helper}

		file = File.new("#{@reportFolder}/preferences-mas.txt", "w+")

		file.write(output)

		file.close

		if File.exist?("#{@reportFolder}/preferences-mas.txt")
			return true
		else
			return false
		end
	end

	def generate_running_processes
		Log.instance.info("[Diagnostics] Generating running process list")
		processes = %x{ps aux}

		process_file = File.new("#{@reportFolder}/running_processes.txt", "w+") << processes
		process_file.close
	
		Log.instance.info("[Diagnostics] Process list saved")

		if File.exist?("#{@reportFolder}/running_processes.txt")
			return true
		else
			return false
		end
	end
end
