require 'helpers'

class Chrome
	attr_reader :version
	attr_reader :profiles
	attr_reader :valid_code_signature
	attr_reader :code_signature_output

	def initialize
		@version = get_version
		@profiles = get_profiles
		@valid_code_signature = Helpers.verify_code_signature("/Applications/Google Chrome.app/")
		@code_signature_output = Helpers.code_signature_output("/Applications/Google Chrome.app/")
	end

	def get_version
		return Helpers.readPlistValueForKeyPath("/Applications/Google\\ Chrome.app/Contents/Info.plist", "CFBundleShortVersionString")
	end

	def get_profiles
		home_folder = File.expand_path("~/")

		return [] unless File.exists?("#{home_folder}/Library/Application Support/Google/Chrome/")

		profile_paths = Dir["#{home_folder}/Library/Application Support/Google/Chrome/**/Preferences"]

		profile_list = []

		profile_paths.each do |profile|
			current_profile = OpenStruct.new
			current_profile.path = profile.gsub!(/Preferences/, "")
			current_profile.name = profile_name(current_profile.path)

			profile_list.push current_profile
		end

		profiles = []
			
		if profile_list != nil
			profile_list.each do |profile|
				temp_profile = OpenStruct.new
				temp_profile.name = profile.name
				temp_profile.path = profile.path
				temp_profile.extensions = installed_extensions(profile.path)
				temp_profile.javascript_enabled = javascript_enabled?(profile.path)
				temp_profile.jse3 = get_jse3_database_information(profile.path)

				profiles.push temp_profile
			end
		end

		return profiles
	end

	def profile_name(profile_path)
		temp_path = profile_path.split("/")
		return temp_path[-1]
	end

	def javascript_enabled?(profile_path)
		pref_path = profile_path + "Preferences"
		pref_path.gsub!(/ /, "\\ ")
		
		search_string = '\"javascript\"\: 2'
		js = `grep \"#{search_string}\" #{pref_path}`

		if js =~ / 2/
			return false
		else
			return true
		end

	end

	def installed_extensions(profile_path)
		return [] unless File.exists?(profile_path)

		extension_folders = Dir["#{profile_path}Extensions/**/manifest.json"]

		extensions = []

		extension_folders.each do |extension|
			temp_manifest = File.open(extension, "r").read
			
			version = $1 if temp_manifest =~ /\"version\"\: \"(.*?)\"/
			name = $1 if temp_manifest =~ /\"name\"\: \"(.*?)\"/

			current_extension = OpenStruct.new

			current_extension.name = name
			current_extension.version = version
			current_extension.path = extension
			current_extension.extension_id = extension.split("/")[-3]

			extensions.push current_extension
			
		end

		return extensions
	end

	def get_jse3_database_information(profile_path)
		jse3 = []

		Log.instance.info("[Chrome JSE3] Profile Path: #{profile_path}")
		database_db_path = "#{profile_path}/databases/Databases.db"

		# TODO: Check if database_db_path exists!
		raw_id_origin = Helpers.sqlite_query(database_db_path, "SELECT id,origin FROM Databases WHERE name = 'OnePassword';")

		entry = raw_id_origin.split("\n")

		entry.each do |extension|
			temp_db = OpenStruct.new
			temp_id, temp_origin = extension.split('|')

			db_path = "#{profile_path}/databases/#{temp_origin}/#{temp_id}"

			if File.exists?(db_path)
				if temp_origin =~ /gkndfifo/
					temp_db.distribution = "Direct"
				elsif temp_origin =~ /hdmbinom/
					temp_db.distribution = "Chrome Web Store"
				else
					temp_db.distribution = "Not Available"
				end
			
				temp_db.path = db_path
				temp_db.origin = temp_origin
				temp_db.schema = get_jse3_schema(db_path)
				temp_db.config = get_jse3_config(db_path)
				temp_db.total_items = get_jse3_total_items(db_path)
				temp_db.total_keys = get_jse3_total_keys(db_path)
				temp_db.unsynced_items = get_jse3_unsynced_items(db_path)

				jse3 << temp_db
			end
		end

		return jse3
	end

	def get_jse3_schema(db_path)
		schema = Helpers.sqlite_query(db_path, ".schema")

		return schema
	end

	def get_jse3_config(db_path)
		config = Helpers.sqlite_query(db_path, "SELECT * FROM config;")

		return config
	end

	def get_jse3_total_items(db_path)
		raw_total_items = Helpers.sqlite_query(db_path, "SELECT count(*) FROM items;")

		return raw_total_items.chomp!.to_i
	end

	def get_jse3_total_keys(db_path)
		raw_total_keys = Helpers.sqlite_query(db_path, "SELECT count(*) FROM keys;")

		return raw_total_keys.chomp!.to_i
	end

	def get_jse3_unsynced_items(db_path)
		raw_unsynced_items = Helpers.sqlite_query(db_path, "SELECT count(*) FROM items WHERE contents_hash = \"777\";")

		return raw_unsynced_items.chomp!.to_i
	end

end

