module Helpers
	def self.osx_version
		sw_vers = `sw_vers`
		
		return $1 if sw_vers =~ /ProductVersion\:\s(.*?)\n/
	end
	# Helpers.readDefaults
	#
	# Provide a domain and a key, returns a string value
	def self.readDefaults(domain, key)
		%x{defaults read #{domain} #{key}}
	end

	def self.findPlistValueForKey(file, key, value="string")
		return "Info.plist not found" unless File.exist?(file)
		result = "Error"

		# Todo: Could handle Errno::ENOENT by $?.exitstatus == 127. 
		begin
			plist = File.read(file)
			regex = %r{<key>#{key}<\/key>$\s*<#{value}>(.*?)<\/#{value}>}
			result = $1 if regex =~ plist
			return result
		rescue ArgumentError => error
			return "Encoding Error"	
		rescue StandardError => error
			return "Standard Error"
		end
	end

	def self.readPlistValueForKeyPath(plist_path, key_path)
		# Todo: Could handle Errno::ENOENT by $?.exitstatus == 127
		begin 
			result = `/usr/libexec/PlistBuddy -c "Print #{key_path}" #{plist_path}`
			return result
		rescue
			return "File Not Found"
		end
	end

	def self.ls_output_for_path(path)
		result = "ls -FaBl@WeH #{path}\n\n"
		expanded_path = File.expand_path(path)
		result << `ls -FaBl@WeH "#{expanded_path}"`	

		return result
	end

	def self.stat_output_for_path(path)
		result = "stat #{path}\n\n"
		expanded_path = File.expand_path(path)
		result << %x{stat "#{expanded_path}"}
		
		return result
	end

	def self.home
		return File.expand_path("~/")
	end
	
	def self.process_started?(executable_name)
		last_char = executable_name[-1].chr
		result = `ps -A | grep #{executable_name[0..-2]}[#{last_char}]`  # e.g. `ps -A | grep librarian[d]` will find /usr/libexec/librariand but avoid the grep command
		return result != ""
	end

	def self.spotlight_search(query)
		"Results of mdfind #{query}\n\n" << %x{mdfind #{query}}
	end

	def self.sqlite_query(file, query)
		%x{sqlite3 "#{file}" "#{query}"}
	end

	def self.verify_code_signature(path)
		expanded_path = File.expand_path(path)

		result = `codesign --verify --verbose "#{expanded_path}" 2>&1`

		if result =~ /satisfies its Designated Requirement/
			true
		else
			false
		end
	end

	def self.code_signature_output(path)
		expanded_path = File.expand_path(path)

		`codesign --verify --verbose "#{expanded_path}" 2>&1`
	end

	def self.booleanForKey(domain, key, default, truthy, invert)
    truthy = Regexp.new(truthy, Regexp::EXTENDED|Regexp::IGNORECASE) unless truthy.is_a? Regexp
    result = case Helpers.readDefaults(domain, key).chomp

    when /does not exist/
        default
    when truthy
        true
    else
        nil
    end

    invert ? !result : result
	end

end

class TrueClass 
	def to_yesno
		return "Yes"
	end
end

class FalseClass 
	def to_yesno
		return "No"
	end
end
