require 'helpers'

class OP3

	# MAS
	attr_reader :mas
	attr_reader :web

	def initialize(safemode)
		@safemode = safemode
		
		@mas = get_mas_data
		@web = get_website_data

	end

	def get_mas_data
		data = OpenStruct.new

		data.data_path = get_mas_data_path
		data.item_count = get_item_count(data.data_path)
		data.tombstone_count = get_tombstone_count(data.data_path)
		data.backups = get_mas_backups

		return data
	end

	def get_website_data
		data = OpenStruct.new

		data.data_path = get_web_data_path
		data.item_count = get_item_count(data.data_path)
		data.tombstone_count = get_tombstone_count(data.data_path)

		# Preferences
		data.lock_when_idle = web_lock_when_idle?
		data.lock_when_idle_timeout = get_web_lock_when_idle_timeout
		data.lock_when_sleeping = web_lock_when_sleeping?
		data.lock_when_screensaver = web_lock_when_screensaver?

		data.backup_path = get_web_backup_path
		data.backups = get_web_backups

		return data
	end

	def get_item_count(path)
		default_folder = "#{path}/data/default"
		opfiles = Dir["#{default_folder}/*[^)].1password"]
		return opfiles.length
	end

	def get_tombstone_count(path)
		default_folder = "#{path}/data/default"
		Log.instance.info("XXX: default_folder = #{default_folder}")
		tombstones = %x{grep -il "tombstone" "#{default_folder}/*.1password" | wc -l}.chomp
		return tombstones
	end

	def get_mas_data_path
		result = Helpers.readDefaults("com.agilebits.onepassword-osx-helper", "DatabasePath")
		result = "~/Library/Containers/com.agilebits.onepassword-osx-helper/Data/Documents/1Password.agilekeychain" if result.nil? or result.length == 0

		return File.expand_path(result)
	end

	def get_mas_backups
		path = File.expand_path("~/Library/Containers/com.agilebits.onepassword-osx/Data/Library/Application Support/1Password/Backups/")

		return Helpers.ls_output_for_path(path)
	end

	def get_web_data_path
		result = Helpers.readDefaults("ws.agile.1Password", "AgileKeychainLocation").chomp
		result = "~/Library/Application Support/1Password/1Password.agilekeychain" if result.nil? or result.length == 0

		return File.expand_path(result)
	end

	def get_web_backup_path
		result = Helpers.readDefaults("ws.agile.1Password", "keychainBackupFolderPath").chomp
		result = "~/Library/Application Support/1Password/Backups" if result.nil? or result.length == 0

		return File.expand_path(result)
	end

	def get_web_backups
		path = get_web_backup_path

		return Helpers.ls_output_for_path(path)
	end

	def web_lock_when_idle?
		result = Helpers.readDefaults("ws.agile.1Password", "LockAgileKeychainOnIdle").chomp
		
		if result == "0"
			return false
		else
			return true
		end
	end

	def get_web_lock_when_idle_timeout
		result = Helpers.readDefaults("ws.agile.1Password", "AgileKeychainLockTimeout").chomp

		return result
	end

	def web_lock_when_sleeping?
		result = Helpers.readDefaults("ws.agile.1Password", "LockAgileKeychainOnSleep").chomp

		Helpers.booleanForKey("ws.agile.1Password", "LockAgileKeychainOnSleep", true, "0", true)

		#if result == "0"
		#	return false
		#else
		#	return true
		#end
	end

	def web_lock_when_screensaver?
		result = Helpers.readDefaults("ws.agile.1Password", "LockAgileKeychainOnScreenSaver").chomp

		if result == "0"
			return false
		else
			return true
		end
	end	

	def web_always_start_op_locked?
		result = Helpers.readDefaults("ws.agile.1Password", "AlwaysStart1PasswordLocked").chomp

		if result == "0"
			return false
		else
			return true
		end
	end

	def web_unlock_all_applications_together?
		result = Helpers.readDefaults("ws.agile.1Password", "UnlockAllAppsTogether").chomp

		if result == "0"
			return true 
		else
			return false
		end
	end

	def web_autosubmit?
		result = Helpers.readDefaults("ws.agile.1Password", "autosubmit").chomp

		if result == "0"
			return false
		else
			return true
		end
	end

	def web_autosave?

	end

	def web_download_icons_and_previews?

	end

	def web_automatic_backups?

	end

	def web_perform_backups_every

	end

	def web_keep_backups

	end

	def web_check_for_updates?

	end

	def web_check_for_updates_every

	end

	def web_include_beta_versions?

	end



end
