require 'helpers'

class OP4
	attr_reader :mas
	attr_reader :website
	attr_reader :versions
	attr_reader :agilekeychains
	attr_reader :agilekeychain_folders
	attr_reader :apps

	def initialize(safemode)
		@safemode = safemode
		@temp_folder = %x{mktemp -d -t onepassworddiag}.chomp!

		@website = get_website_data
		@mas = get_mas_data
		@versions = get_versions
		@agilekeychains = find_agilekeychains
		@agilekeychain_folders = find_agilekeychain_folders
		@apps = find_op_apps
	end

	def get_mas_data
		data_path = File.expand_path("~/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Data")
		backups = File.expand_path("~/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Backups")
		app_domain = "com.agilebits.onepassword-osx"
		helper_domain = "2BUA8C4S2C.com.agilebits.onepassword-osx-helper"

		data = get_data_for_distribution(app_domain, helper_domain, data_path, backups)
		data.helper_plist_found = mas_helper_plist_found?
		data.app_plist_found = mas_app_plist_found?

		return data
	end

	def get_website_data
		data_path = File.expand_path("~/Library/Application Support/1Password\ 4/Data")
		backups = File.expand_path("~/Library/Application Support/1Password\ 4/Backups")
		app_domain = "com.agilebits.onepassword4"
		helper_domain = "2BUA8C4S2C.com.agilebits.onepassword4-helper"

		data = get_data_for_distribution(app_domain, helper_domain, data_path, backups)	
		data.helper_plist_found = web_helper_plist_found?
		data.app_plist_found = web_app_plist_found?

		return data
	end

	def get_data_for_distribution(app_domain, helper_domain, data_path, backups)

		data = OpenStruct.new
		data.keep_helper_running = keep_helper_running?(app_domain)
		data.show_status_item = show_status_item_enabled?(helper_domain)
		data.show_rich_icons = show_rich_icons_enabled?(helper_domain)
		data.show_item_counts = show_item_counts_enabled?(helper_domain)		

		data.conceal_passwords = conceal_passwords?(helper_domain)		
		data.lock_on_sleep = lock_on_sleep?(helper_domain)
		data.lock_on_screensaver = lock_on_screensaver?(helper_domain)
		data.lock_on_main_app_closed = lock_on_main_app_closed_enabled?(app_domain)
		data.lock_on_user_switch = lock_on_user_switch?(helper_domain)
		data.lock_on_idle = lock_on_idle?(helper_domain)
		data.lock_timeout = lock_timeout(helper_domain)
		data.clear_pasteboard = clear_pasteboard?(helper_domain)
		data.pasteboard_timeout = pasteboard_timeout(helper_domain)

		data.watchtower_enabled = watchtower_enabled?(helper_domain)

		data.auto_submit = auto_submit_enabled?(helper_domain)
		data.animate_fill = animate_fill_enabled?(helper_domain)
		data.lenient_url_matching = lenient_url_matching?(helper_domain)
		data.auto_save = auto_save_enabled?(helper_domain)

		data.show_copy_uuid = show_copy_uuid_enabled?(app_domain)
		data.show_copy_json = show_copy_json_enabled?(app_domain)
		data.third_party_integration = third_party_integration_enabled?(app_domain)
		data.numbers_and_symbols_sorted_first = numbers_and_symbols_sorted_first?(app_domain)
		
		data.automatically_check_for_updates = automatically_check_for_updates?(helper_domain)
		data.include_beta_builds = include_beta_builds?(helper_domain)

		data.open_urls_in = open_urls_in?(helper_domain)

		data.backups_list = backups_list(backups)
		data.total_unsaved_changes = get_total_unsaved_changes(data_path)

		FileUtils.cp_r(data_path, @temp_folder) if File.exists?(data_path)

		sqlite = "#{@temp_folder}/Data/OnePassword.sqlite"

		data.number_of_profiles = number_of_profiles(sqlite)
		data.profiles = profiles(sqlite, helper_domain)

		FileUtils.rm_rf("#{@temp_folder}/Data/")

		return data
	end
	
	def get_versions
		locations = [
					"/Applications/1Password\ 4.app", 
					"/Applications/1Password\ 5.app",
					"/Applications/1Password.app", 
					"/Applications/1Password.localized/1Password.app", 
					"#{HOME}/Applications/1Password.app", 
					"#{HOME}/Applications/1Password\ 4.app",
					"#{HOME}/Applications/1Password\ 5.app"
		]

		versions = []

		begin
			locations.each do |location|
				temp_location = OpenStruct.new
				temp_location.version = Helpers.findPlistValueForKey("#{location}/Contents/Info.plist", "CFBundleShortVersionString")
				temp_location.build = Helpers.findPlistValueForKey("#{location}/Contents/Info.plist", "CFBundleVersion")
				temp_location.location = location
				temp_location.valid_code_signature = Helpers.verify_code_signature(location)
				temp_location.code_signature_output = Helpers.code_signature_output(location)
				temp_location.plist_found = File.exist?("#{location}/Contents/Info.plist")

				bundle_id = Helpers.findPlistValueForKey("#{location}/Contents/Info.plist", "CFBundleIdentifier")
				temp_location.bundle_id = bundle_id
				if bundle_id == "com.agilebits.onepassword-osx" # 1Password 3.9, 4.x, and 5.x 
					temp_location.distribution = "Mac App Store"
				elsif bundle_id == "com.agilebits.onepassword4" # 1Password 4.x and 5.x 
					temp_location.license = is_licensed?
					temp_location.distribution = "Website"
				elsif bundle_id == "ws.agile.1Password"
					temp_location.distribution = "Website"
				else
					temp_location.distribution = "Unable to Determine"
				end

				versions.push temp_location if File.exists?(location)
			end
		end

		return versions
	end

	def keep_helper_running?(domain)
		result = Helpers.readDefaults(domain, "KeepHelperRunning")
		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# Sync Settings
	#
	#
	
	def dropbox_enabled?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper DropboxSyncEnabled`.chomp!
		result = Helpers.readDefaults(domain, "DropboxSyncEnabled")

		if result.chomp! == "1" 
			return true 
		else 
			return false
		end
	end

	def icloud_enabled?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper UbiquitySyncEnabled`.chomp!
		result = Helpers.readDefaults(domain, "UbiquitySyncEnabled")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# Security Settings
	#
	#
	
	def conceal_passwords?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper ConcealPasswords`.chomp!
		result = Helpers.readDefaults(domain, "ConcealPasswords")

		if result.chomp! == "1"
			return true
		else
			return false
		end

	end

	def clear_pasteboard?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper ClearPasteboardAfterTimeout`.chomp!
		result = Helpers.readDefaults(domain, "ClearPasteboardAfterTimeout")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def pasteboard_timeout(domain)
		#`defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper PasteboardClearTimeout`.chomp!
		Helpers.readDefaults(domain, "PasteboardClearTimeout").chomp!
	end

	def lock_on_idle?(domain)
		# LockOnIdle
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper LockOnIdle`.chomp!
		result = Helpers.readDefaults(domain, "LockOnIdle")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def lock_on_screensaver?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper LockOnScreenSaver`.chomp!
		result = Helpers.readDefaults(domain, "LockOnScreenSaver")

		if result.chomp! == "1"
			return true
		else
			return false
		end

	end

	def lock_on_sleep?(domain)
		#result = `defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper LockOnSleep`.chomp!
		result = Helpers.readDefaults(domain, "LockOnSleep")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def lock_on_main_app_closed_enabled?(app_domain)
		result = Helpers.readDefaults(app_domain, "LockOnMainAppExit")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def lock_on_user_switch?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "LockOnUserSwitch")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def lock_timeout(helper_domain)
		#`defaults read 2BUA8C4S2C.com.agilebits.onepassword4-helper LockTimeout`.chomp!	
		Helpers.readDefaults(helper_domain, "LockTimeout").chomp!
	end

	def watchtower_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "watchtowerService")
		
		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# show_rich_icons_enabled?
	#		Arguments: helper_domain - specify the domain for defaults read
	#		Returns: BOOL
	def show_rich_icons_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "ShowRichIcons")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def show_item_counts_enabled?(app_domain)
		result = Helpers.readDefaults(app_domain, "ShowItemCounts")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# show_status_item_enabled?
	#		Arguments: helper_domain - specify the domain for defaults read
	#		Returns: BOOL
	def show_status_item_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "ShowStatusItem")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# auto_submit_enabled?
	#		Arguments: helper_domain - specify the domain for defaults read
	#		Returns: BOOL
	def auto_submit_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "autosubmit")

		# Check for 0 here because it's enabled by default, 
		#		and default doesn't show up in plist without first changing it
		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	# open_urls_in?(helper_domain)
	#		Arguments: helper_domain - specify the domain for defaults read
	#		Returns: OpenStruct
	def open_urls_in?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "GoAndFillOpenInTabMode")

		response = OpenStruct.new

		# Check for 1 and 3 first, else will be 2 (default)
		if result.chomp! == "1"
			response.text = "Current Tab"
			response.number = 1
		elsif result.chomp! == "3"
			response.text = "New Window"
			response.number = 3
		else
			response.text = "New Tab"
			response.number = 2
		end

		return response
	end

	def animate_fill_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "animateFill")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def lenient_url_matching?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "lenientURLMatch")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def auto_save_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "autosave")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def third_party_integration_enabled?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "Enable3rdPartyIntegration")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def numbers_and_symbols_sorted_first?(helper_domain)
		result = Helpers.readDefaults(helper_domain, "HashSectionIsSortedFirst")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def show_copy_json_enabled?(app_domain)
		result = Helpers.readDefaults(app_domain, "ShowCopyJSONItemMenu")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def show_copy_uuid_enabled?(app_domain)
		result = Helpers.readDefaults(app_domain, "ShowCopyUUIDItemMenu")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def automatically_check_for_updates?(app_domain)
		result = Helpers.readDefaults(app_domain, "CheckForSoftwareUpdatesEnabled")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	def include_beta_builds?(app_domain)
		result = Helpers.readDefaults(app_domain, "CheckForSoftwareUpdatesIncludeBetas")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def backups_list(backups_path)
		Helpers.ls_output_for_path(backups_path)
	end

	def find_op_apps
		return "" if @safemode
		Helpers.spotlight_search("1Password kind:Application")
	end

	def find_agilekeychains
		return "" if @safemode
		Helpers.spotlight_search("kind:Agile Keychain")
	end

	def find_agilekeychain_folders
		return "" if @safemode
		Helpers.spotlight_search(".agilekeychain_folder")
	end

	def number_of_profiles(sqlite_path)
		return 0 unless File.exists?(sqlite_path)

		Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM profiles;")
	end

	def profiles(sqlite_path, helper_domain)
		return [] unless File.exists?(sqlite_path)

		raw_profiles = Helpers.sqlite_query(sqlite_path, "SELECT id, uuid, profile_name, updated_at, created_at, trashed, iterations FROM profiles;") 
		profiles = raw_profiles.split("\n").collect do |profile|
			columns = profile.split('|')

			temp_profile = OpenStruct.new
			temp_profile.number = columns[0]
			temp_profile.uuid = columns[1]

			profile_name = columns[2]

			temp_profile.profile_name = profile_name #columns[2]
			temp_profile.updated_at = columns[3]
			temp_profile.readable_updated_at = Time.at(Integer(columns[3])).strftime("%m/%d/%Y")
			temp_profile.created_at = columns[4]
			temp_profile.trashed = columns[5]
			temp_profile.iterations = columns[6]

			if helper_domain =~ /onepassword-osx-helper/
				plist = "#{HOME}/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Preferences/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.plist"
			else
				plist = "#{HOME}/Library/Preferences/2BUA8C4S2C.com.agilebits.onepassword4-helper.plist"
			end

			temp_sync_info = sync_information_for_profile(sqlite_path, plist, temp_profile.number)
			if temp_sync_info
				if temp_sync_info.enabled == true
					temp_profile.sync_enabled = true
					temp_profile.sync_location_type = temp_sync_info.location_type
					temp_profile.sync_keychain_type = temp_sync_info.keychain_type
					temp_profile.sync_local_profile_uuid = temp_sync_info.local_profile_uuid
					temp_profile.sync_keychain_path = temp_sync_info.keychain_path
					temp_profile.sync_keychain_contents = temp_sync_info.keychain_contents
				else
					temp_profile.sync_enabled = false
				end
			end

			temp_profile.total_items = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM items WHERE profile_id = #{temp_profile.number};")
			temp_profile.non_trashed_items = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM items WHERE profile_id = #{temp_profile.number} AND trashed = 0;")
			temp_profile.trashed_items = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM items WHERE profile_id = #{temp_profile.number} AND trashed = 1 AND category_uuid <> \'099\';")
			temp_profile.tombstoned_items = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM items WHERE profile_id = #{temp_profile.number} AND category_uuid = \'099\';")

			temp_profile.total_folders = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM folders WHERE profile_id = #{temp_profile.number};")
			temp_profile.non_trashed_folders = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM folders WHERE profile_id = #{temp_profile.number} AND trashed = 0;")
			temp_profile.trashed_folders = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM folders WHERE profile_id = #{temp_profile.number} AND trashed = 1;")

			temp_profile.total_attachments = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM item_attachments WHERE profile_id = #{temp_profile.number};")
			temp_profile.non_trashed_attachments = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM item_attachments WHERE profile_id = #{temp_profile.number} AND trashed = 0;")
			temp_profile.trashed_attachments = Helpers.sqlite_query(sqlite_path, "SELECT COUNT(*) FROM item_attachments WHERE profile_id = #{temp_profile.number} AND trashed = 1;")

			temp_profile
		end
	end

	# sync_information_for_profile
	#		Arguments: 
	#			filename - (string) path to sqlite file
	#			plist - (string) path to helper plist
	#			profile_id - (integer) value of profile
	#		Returns: OpenStruct
	def sync_information_for_profile(filename, plist, profile_id)
		sync_table_data_raw = Helpers.sqlite_query(filename, "SELECT keychain_type, local_profile_uuid, location_type FROM sync_sources WHERE profile_id = #{profile_id}").chomp!

		sync_source = OpenStruct.new

		if sync_table_data_raw == nil 
			sync_source.enabled = false
			return sync_source
		else
			sync_source.enabled = true
		end

		columns = sync_table_data_raw.split("|")

		sync_source.keychain_type = columns[0]
		sync_source.local_profile_uuid = columns[1]
		sync_source.keychain_path = columns[2]

		#command = "/usr/libexec/PlistBuddy -c"

		#sync_source.keychain_path = `#{command} 'Print SyncMonitor_#{sync_source.local_profile_uuid}:kKeychainPath' "#{plist}"`

		if sync_source.keychain_type == "Agile"
			sync_source.keychain_contents = Helpers.ls_output_for_path("#{sync_source.keychain_path.chomp}/data/default")
		else
			sync_source.keychain_contents = nil
		end
		
		return sync_source

	end

	def is_licensed?
		license = File.expand_path("~/Library/Application\ Support/1Password\ 4/License/License.onepassword-license")
		mas_receipt = File.expand_path("~/Library/Application\ Support/1Password\ 4/App\ Store\ Receipts/receipt")

		if File.exists?(license)
			order_number = Helpers.findPlistValueForKey(license, "OrderNumber", "integer")
		elsif File.exists?(mas_receipt)
			"MAS License"
		else
			"No license found"
		end
	end

	def mas_helper_plist_found?
		File.exist?(File.expand_path("~/Library/Containers/2BUA8C4S2C.com.agilebits.onepassword-osx-helper/Data/Library/Preferences/2BUA8C4S2C.com.agilebits.onepassword-osx-helper.plist"))
	end

	def mas_app_plist_found?
		File.exist?(File.expand_path("~/Library/Containers/com.agilebits.onepassword-osx-old/Data/Library/Preferences/com.agilebits.onepassword-osx.plist"))
	end

	def web_helper_plist_found?
		File.exist?(File.expand_path("~/Library/Preferences/2BUA8C4S2C.com.agilebits.onepassword4-helper.plist"))
	end

	def web_app_plist_found?
		File.exist?(File.expand_path("~/Library/Preferences/com.agilebits.onepassword4.plist"))
	end

	def get_total_unsaved_changes(data_path)
		results = Dir.glob("#{data_path}/*.changes")
		return results.count
	end
end
