require 'helpers'

class Opera
	attr_reader :version
	attr_reader :extensions
	attr_reader :javascript_enabled
	attr_reader :valid_code_signature
	attr_reader :code_signature_output

	def initialize
		@version = get_version
		@extensions = get_extensions
		@javascript_enabled = javascript_enabled?
		@valid_code_signature = Helpers.verify_code_signature("/Applications/Opera.app/")
		@code_signature_output = Helpers.code_signature_output("/Applications/Opera.app/")
	end

	def get_version
		return Helpers.readPlistValueForKeyPath("/Applications/Opera.app/Contents/Info.plist", "CFBundleVersion")
	end

	def get_extensions
		extension_folders = Dir["#{HOME}/Library/Application\\ Support/com.operasoftware.opera/Extensions/**/manifest.json"]

		extensions = []

		extension_folders.each do |extension|
			temp_manifest = File.open(extension, "r").read
			
			manifest = JSON.parse(temp_manifest)

			current_extension = OpenStruct.new
			current_extension.name = manifest["name"]
			current_extension.version = manifest["version"]
			current_extension.extension_id = extension.split("/")[-3]

			extensions.push current_extension
		end

		return extensions
	end

	def javascript_enabled?
		pref_path = File.expand_path("~/Library/Application\\ Support/com.operasoftware.Opera/Preferences")
		
		search_string = '\"javascript\"\: 2'
		js = `grep \"#{search_string}\" #{pref_path}`

		if js =~ / 2/
			return false
		else
			return true
		end

	end

end
