require 'helpers'
require 'log'

class Ubiquity
	attr_reader :librariand_started
	attr_reader :contents_of_release_container
	attr_reader :sanitized_profile_js
	attr_reader :contents_of_beta_container
	attr_reader :docs_and_data_enabled
	attr_reader :mobile_docs_stat

	def initialize
		@release_container_folder = File.expand_path("~/Library/Mobile\ Documents/2BUA8C4S2C\~com\~agilebits\~onepassword/onepassword_data/default")
		@beta_container_folder = File.expand_path("~/Library/Mobile\ Documents/8RS32JY83C\~com\~agilebits\~beta\~onepassword/onepassword_data/default")

		@librariand_started = Helpers.process_started?("librariand")
		
		@contents_of_release_container = get_contents_of_release_container
		@sanitized_profile_js = get_sanitized_profile_js

		@contents_of_beta_container = get_contents_of_beta_container
		@docs_and_data_enabled = docs_and_data_enabled?

		@mobile_docs_stat = get_mobile_docs_stat
	end

private

	def get_contents_of_release_container
		return Helpers.ls_output_for_path(@release_container_folder)
	end
	
	def get_contents_of_beta_container
		return Helpers.ls_output_for_path(@beta_container_folder)
	end

	def get_sanitized_profile_js
		profile_file = File.join(@release_container_folder, "profile.js")

		return "profile.js does not exist" unless File.exists?(profile_file)

		begin
			contents = File.read(profile_file)
		rescue => e
			return "profile.js could not be read. This could be due to not being completely downloaded yet, or it might be corrupt."
			Log.instance("[Ubiquity] Exception Message: #{e.message}")
		end
		
		# TODO: This probably doesn't need to be in a begin/rescue now that we resolved the issue with Ruby 1.8 vs Ruby 2.0
		begin
			contents.gsub!(/"masterKey"\:".*?"/, '"masterKey":"*****"')
			contents.gsub!(/"overviewKey"\:".*?"/, '"overviewKey":"*****"')
			contents.gsub!(/"salt"\:".*?"/, '"overviewKey":"*****"')
			contents.gsub!(/"passwordHint"\:".*?"/, '"passwordHint":"*****"')
		rescue => e
			Log.instance.warn("[Ubiquity] Exception in Ubiquity!")
			Log.instance.warn("[Ubiquity] Exception Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[Ubiquity] Exception Message: #{e.message}")
			Log.instance.warn("[Ubiquity] Exception Inspect: #{e.inspect}")
		end			

		return contents
	end

	def docs_and_data_enabled?
		if File.exists?("#{HOME}/Library/Mobile\ Documents/")
			return true
		else
			if File.exists?("#{HOME}/Library/Preferences/MobileMeAccounts.plist")
				return false
			end
		end
		
		return nil		
	end

	def get_mobile_docs_stat
		%x{stat "#{HOME}/Library/Mobile\ Documents/"}
	end

	# Returns: True, False, nil (not setup, missing file)
	def broken_docs_and_data_enabled?
		command = "/usr/libexec/Plistbuddy -c "
		plist = "#{HOME}/Library/Preferences/MobileMeAccounts.plist"
		enabled = "false" 
		
		unless File.exists?(plist)
			Log.instance.warn("[Ubiquity] ~/Library/Preferences/MobileMeAccounts.plist not found")
			return nil 
		end

		raw_dict_count = `/usr/libexec/PlistBuddy -c "Print Accounts:0:Services" #{plist} | grep "Dict" | wc -l`
		dict_count = raw_dict_count.gsub(/\s+/, "").to_i

		dict_count.times do |index|

				result = `#{command} 'Print Accounts:0:Services:#{index}:Name' #{plist} 2>&1`
				if result.chomp! == "MOBILE_DOCUMENTS"
					enabled = `#{command} 'Print Accounts:0:Services:#{index}:Enabled' #{plist}`
					Log.instance.info("[Ubiquity] #{index} was MOBILE_DOCUMENTS. Enabled = #{enabled.inspect}")
				else
					Log.instance.info("[Ubiquity] #{index} was not MOBILE_DOCUMENTS. Result = #{result.inspect}")
				end
		end

		if enabled =~ /true/
				return true
		else
				return false
		end
	end


end
