require 'erb'
require 'date'

require 'log'

class ReportController
	def initialize(data, output_file, template_file, component_string, safemode)
		@data = data 
		@template = template_file
		@component_string = component_string
		@output_file = output_file
		@safemode = safemode
		@template_folder = File.join(File.dirname(__FILE__), "templates/")

		Log.instance.info("[System] Beginning Processing of #{@component_string}")

		# check if template exists
		if File.exist?(template_file)
			Log.instance.info("[#{@component_string}] Template: #{template_file} exists")
		
			Log.instance.info("[#{component_string}] Creating ERB module")
			@erb = ERB.new(File.read(@template))
			Log.instance.info("[#{@component_string}] Gathering data for template")
			gather
			run
		end

	end

	def run
		begin
			Log.instance.info("[#{@component_string}] Generating report")
			result = @erb.result binding
			Log.instance.info("[#{@component_string}] Saving generated report: #{@output_file}")
			report = File.new(@output_file, "w+") << result
			report.close
		rescue NameError => error
			Log.instance.warn("[#{@component_string}] Undeclared variable: #{error.name}")
			Log.instance.warn("[#{@component_string}] Backtrace \n\n #{error.backtrace.join("\n")}\n\n")			
			Log.instance.warn("[#{@component_string}] Message: #{error.message}")
			Log.instance.warn("[#{@component_string}] Inspect: #{error.inspect}")
		rescue StandardError => e
			Log.instance.warn("[#{@component_string}] Template Generation Failed")
			Log.instance.warn("[#{@component_string}] Exception class: #{e.class}")			
			Log.instance.warn("[#{@component_string}] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[#{@component_string}] Message: #{e.message}")
			Log.instance.warn("[#{@component_string}] Inspect: #{e.inspect}")
		end
	end

	def gather

	end

	# Returns a string
	def render_partial(template)
		Log.instance.info("[#{@component_string}] Begin rendering of partial: #{template}")
		filename = "#{@template_folder}/_#{template}.html.erb"

		if File.exists?(filename)
			# Create ERB instance
			partial_erb = ERB.new(File.read(filename))

			begin
				Log.instance.info("[#{@component_string}] Rendering partial: #{template}")
				result = partial_erb.result binding
			rescue NameError => error
				Log.instance.warn("[#{@component_string}] Undeclared variable: #{error.name}")
				Log.instance.warn("[#{@component_string}] Backtrace \n\n #{error.backtrace.join("\n")}\n\n")			
				Log.instance.warn("[#{@component_string}] Message: #{error.message}")
				Log.instance.warn("[#{@component_string}] Inspect: #{error.inspect}")
			rescue StandardError => e
				Log.instance.warn("[#{@component_string}] Template Generation Failed")
				Log.instance.warn("[#{@component_string}] Exception class: #{e.class}")			
				Log.instance.warn("[#{@component_string}] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
				Log.instance.warn("[#{@component_string}] Message: #{e.message}")
				Log.instance.warn("[#{@component_string}] Inspect: #{e.inspect}")
			end

			return result
		end
	end

	def menu_item_link(path, label, enabled)
		if enabled 
			return "<li><a href=\"#{path}\">#{label}</a></li>"
		else
			return "<li class=\"disabled\"><a href=\"#\">#{label}</a></li>"
		end
	end
end
