require 'log'

class TestController
	def initialize(data)
		# Initial ivars
		@log_string = self.class.name
		@test_folder = self.class.name
		@data = data

	end

	def gather
		Log.instance.info("[#{@log_string}] Gathering Tests From Manifest...")

		# Tests Array
		@tests = []

		# Read each item from the manifest file
		f = File.open(File.join(File.dirname(__FILE__), 'manifest'))
		f.each_line do |line|
			Log.instance.info("[#{@log_string}] ...Adding tests/#{line.strip}")
			@tests.push line.strip
		end
	end

	def run
		self.gather

		Log.instance.info("[#{@log_string}] Running Tests...")

		# Results Array
		results = []

		# Enumerate through each test
		@tests.each do |currentTest|

			begin

				Log.instance.info("[#{@log_string}] ... Trying #{currentTest}")
				require "tests/#{@test_folder}/#{currentTest}"

				# Create a new test object for the current test
				testObject = Kernel.const_get(currentTest).new

				# Run Setup on the test first
				testObject.setup

				# Assign Data
				testObject.assign_data(@data)

				# Execute the test
				testObject.runTest

				# Gather Results
				results.push testObject.result

			rescue LoadError
				Log.instance.warn("[#{@log_string}] ... ERROR. Unable to load #{currentTest}")
			end
		end

		return results
	end
end
